package tests;

import java.awt.geom.Point2D;
import javax.swing.JFrame;

import fr.lri.swingstates.canvas.CShape;
import fr.lri.swingstates.canvas.CStateMachine;
import fr.lri.swingstates.canvas.Canvas;
import fr.lri.swingstates.sm.State;
import fr.lri.swingstates.sm.Transition;
import fr.lri.swingstates.sm.transitions.Drag;
import fr.lri.swingstates.sm.transitions.Press;
import fr.lri.swingstates.sm.transitions.Release;

public class DragAndDropApp extends Canvas{

	// distance minimal pour une action de drag-and-drop 
	private final static double MIN_DISTANCE = 10;
	
	// Une seule Machine  tats pour toutes les interaction 
	CStateMachine m1 = new CStateMachine(){
		
		// Variables globales partages parmi tous les tats
		private Window activeWindow = null;
		private Icon selectedIcon = null;
		private ZombieIcon zombieIcon;
		private Point2D p_;
		
		// Quelques methodes globales
		private void activateWindow(Window win){
			if(win != activeWindow){
				if(activeWindow != null) activeWindow.setActive(false);
				if(win != null) win.setActive(true);
				activeWindow = win;
			}
		}
		
		private void selectIcon(Icon icon){
			if(icon != selectedIcon){
				if(selectedIcon != null) selectedIcon.setSelected(false);
				icon.setSelected(true);
				selectedIcon = icon;
			}
		}
		

		// Premier tat
		public State s1 = new State(){	
				
			// Appuyer sur la barre de la fentre 
			Transition t11 = new PressOnShape(BUTTON1, "=> s2"){
				public boolean guard(){
					return (getShape() instanceof WindowBar);
				}
				
				public void action(){
					WindowBar wbar = (WindowBar)getShape();
					activateWindow(wbar.getWindow());
					
					p_ = getPoint();
					
					if(selectedIcon != null) selectedIcon.setSelected(false);
					selectedIcon = null;
				}
			};
			
			// Appuyer sur le corps de la fentre 
			Transition t12 = new PressOnShape(BUTTON1){
				public boolean guard(){
					return (getShape() instanceof WindowBody);
				}
				
				public void action(){
					WindowBody wbody = (WindowBody)getShape();
					activateWindow(wbody.getWindow());
					
					if(selectedIcon != null) selectedIcon.setSelected(false);
					selectedIcon = null;
				}
			};
			
			// Appuyer sur une icne
			Transition t13 = new PressOnShape(BUTTON1, "=> s3"){
				public boolean guard(){
					return (getShape() instanceof Icon);
				}
				
				public void action(){
					Icon icon = (Icon)getShape();
					
					activateWindow(icon.getParentWindow());
					selectIcon(icon);
					p_ = getPoint();
				}
			};
			
			// Appuyer sur le vide
			Transition t14 = new Press(BUTTON1){
				public void action(){
					if(activeWindow != null) activeWindow.setActive(false);
					activeWindow = null;
					
					if(selectedIcon != null) selectedIcon.setSelected(false);
					selectedIcon = null;
				}
			};
			
		};
		
		// 2me tat : Deplacement d'une fentre
		public State s2 = new State(){
			
			// Dragging sur la barre de la fentre 
			Transition t21 = new Drag(BUTTON1){
				public void action(){
					Point2D p = getPoint();
					activeWindow.translateBy(p.getX() - p_.getX(), p.getY() - p_.getY());
					p_ = p;
				}
			};
			
			Transition t22 = new Release(BUTTON1, "=> s1"){
			};
		};
		
		
		// 3me tat : Dragging sue une icne : le dplacement n'a pas encore depass un seuil de MIN_DISTANCE.
		public State s3 = new State(){
						
			// Le dplacement dpasse la distance de de MIN_DISTANCE. Une icne "zombie" est cre pour guider l'action "drag-drop" 
			Transition t31 = new Drag(BUTTON1, "=> s4"){

				public boolean guard(){
					return (getPoint().distance(p_) > MIN_DISTANCE);
				}
				
				public void action(){
					zombieIcon = new ZombieIcon(getPoint().getX(), getPoint().getY(), selectedIcon);
					addShape(zombieIcon);
					zombieIcon.aboveAll();
				}
			};
			
			
			Transition t32 = new Release(BUTTON1, "=> s1"){};
		};
		
		

		// tat de drag-and-drop
		public State s4 = new State(){

			// Dragging
			Transition t41 = new Drag(BUTTON1){
				public void action(){
					zombieIcon.translateTo(getPoint().getX(), getPoint().getY());
				}
			};
			
			// Entre dans une nouvelle fentre
			Transition t42 = new EnterOnShape(){
				public boolean guard(){
					CShape shape = getShape();
					// il faut prvoir le cas qu'on entre dans une icne de la fentre.... 
					return (shape instanceof WindowBody || shape instanceof Icon && ((Icon)shape).getParentWindow() != null);
				}
				
				public void action(){
					Window win;
					
					if(getShape() instanceof Icon) 
						win = ((Icon)getShape()).getParentWindow();
					else win = ((WindowBody)getShape()).getWindow();
					 
					activateWindow(win);
					zombieIcon.aboveAll();
				}	
			};
			
			
			// Sortie d'une fentre... ncessaire dans le cas qu'on est plus dans une fentre
			Transition t43 = new LeaveOnShape(){
				public boolean guard(){
					CShape shape = getShape();
					// il faut prvoir le cas qu'on sort d'une icne de la fentre.... 
					return (shape instanceof WindowBody || shape instanceof Icon && ((Icon)shape).getParentWindow() != null);
				}
				
				public void action(){
					Window win;
					
					if(getShape() instanceof Icon) 
						win = ((Icon)getShape()).getParentWindow();
					else win = ((WindowBody)getShape()).getWindow();
					
					win.setActive(false);
					activeWindow = null;
				}
			};
			
			// L'action "drag-and-drop" est complt. L'icne est deplace et le "zombie" est dtruit. 
			Transition t44 = new Release(BUTTON1, "=> s1"){
				public void action(){
					selectedIcon.setParentWindow(activeWindow);
					selectedIcon.translateTo(getPoint().getX(), getPoint().getY());				
					selectedIcon.aboveAll();
	
					removeShape(zombieIcon);
				}
			};
		};
		
	};
	
	
	final static int ICON_WIDTH = 30;
	final static int ICON_HEIGHT = 40;
	
	public DragAndDropApp(){
		super(1200, 800);
		
		Icon icon1, icon2, icon3, icon4;
		
		icon1 = new Icon(600, 500, ICON_WIDTH, ICON_HEIGHT);
		addShape(icon1);
		icon2 = new Icon(200, 400, ICON_WIDTH, ICON_HEIGHT);
		addShape(icon2);
		icon3 = new Icon(700, 600, ICON_WIDTH, ICON_HEIGHT);
		addShape(icon3);
		icon4 = new Icon(100, 50, ICON_WIDTH, ICON_HEIGHT);
		addShape(icon4);
		
		Window w1, w2, w3;
		w1 = new Window(this, 100, 100, 400, 300);
		w2 = new Window(this, 600, 100, 400, 300);
		w3 = new Window(this, 400, 300, 400, 300);
			
		addShape(w1);
		addShape(w2);
		addShape(w3);
		
		
		m1.attachTo(this);
	}
	
	
	public static void main(String[] args){
		JFrame frame = new JFrame();
		DragAndDropApp app = new DragAndDropApp();
		frame.getContentPane().add(app);
		frame.pack();
		frame.setVisible(true);
	}
	
}
